package ru.ulstu.extractor.ts.creator.db;

import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.gitrepository.model.GitRepository;
import ru.ulstu.extractor.ts.creator.AbstractTimeSeriesCreator;
import ru.ulstu.extractor.ts.model.TimeSeries;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;

import java.util.Comparator;
import java.util.List;

import static ru.ulstu.extractor.ts.util.TimeSeriesDateMapper.mapTimeSeriesToInterval;

public abstract class DBTimeSeriesCreator extends AbstractTimeSeriesCreator {
    /**
     * Извлечь список точек временных рядов
     *
     * @param gitRepository
     * @param branchName
     * @return
     */
    public abstract List<TimeSeries> getTimeSeries(GitRepository gitRepository, String branchName);

    /**
     * Сохранить извлеченные временные ряды
     *
     * @param gitRepository
     * @param branch
     */
    public void addTimeSeries(GitRepository gitRepository, Branch branch) {
        // извлеченные временных рядов
        List<TimeSeries> timeSeries = getTimeSeries(gitRepository, branch.getName());

        // сгруппированные по временным интервалам точки временных рядов
        timeSeries.forEach(ts -> ts.setValues(mapTimeSeriesToInterval(getTimeSeriesService().getTimeSeriesInterval(), ts.getValues())));
        getTimeSeriesService().save(sortTimeSeries(timeSeries), branch);
    }

    private List<TimeSeries> sortTimeSeries(List<TimeSeries> timeSeries) {
        for (TimeSeries ts : timeSeries) {
            ts.getValues().sort(Comparator.comparing(TimeSeriesValue::getDate));
        }
        return timeSeries;
    }
}
